#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <libosso.h>

#include <gtk/gtk.h>
#include <hildon/hildon.h>

#include <rtcom-eventlogger/eventlogger.h>

#define SERVICE_ID "RTCOM_EL_SERVICE_SMS"
#define SMS_INBOUND "RTCOM_EL_EVENTTYPE_SMS_INBOUND"
#define SMS_OUTBOUND "RTCOM_EL_EVENTTYPE_SMS_OUTBOUND"

typedef struct _MessageStruct MessageStruct;

struct _MessageStruct {
	char *service_id;
	char *event_type_id;
	time_t date;
	int is_read;
	char *local_uid;
	char *local_name;
	char *remote_uid;
	char *remote_name;
	char *remote_ebook_uid;
	char *group_uid;
	char *free_text;
};

int main (int argc, char **argv)
{
	osso_context_t *osso_context;
	RTComEl *el = NULL;
	RTComElIter *iter = NULL;
	RTComElQuery *query;
	GError *error = NULL;
	gboolean exist = FALSE;
	char buf[512];
	FILE *fd;

	if (argc != 2) {
		printf ("Usage: smsexporter <output csv file>\n");
		printf ("  ex. smsexporter backup.csv\n");
		return 0;
	}

	hildon_gtk_init (&argc, &argv);

	/* Initialize osso */
	osso_context = osso_initialize ("smsexporter", "0.2", TRUE, NULL);
	if (osso_context == NULL) {
		g_critical ("Error initializing osso");
		return 1;
	}

	/* Init abook */
	if (!osso_abook_init (&argc, &argv, osso_context)) {
		g_critical ("Error initializing libosso-abook");
		goto exit;
	}

	el = rtcom_el_new ();

	fd = fopen (argv[1], "w");
	if (!fd) {
		printf ("Error openning file: %s\n", argv[1]);
		return 1;
	}

	query = rtcom_el_query_new (el);
	rtcom_el_query_prepare (query, "service", SERVICE_ID, RTCOM_EL_OP_EQUAL, NULL);
	iter = rtcom_el_get_events (el, query);

	for (exist = rtcom_el_iter_first (iter); exist == TRUE; exist = rtcom_el_iter_next (iter)) {
		RTComElEvent *ev = NULL;
		struct tm *date;

		ev = rtcom_el_event_new ();
		if (rtcom_el_iter_get (iter, ev)) {
			date = localtime (&RTCOM_EL_EVENT_GET_FIELD (ev, start_time));

			if (!strcmp (RTCOM_EL_EVENT_GET_FIELD (ev, event_type), SMS_INBOUND)) {
				fprintf (fd, "sms;deliver;\"%s\";\"\";\"\";\"%4d.%2d.%2d %2d:%2d\";\"\";\"%s\"\n",
					 RTCOM_EL_EVENT_GET_FIELD (ev, remote_uid),
					 date->tm_year + 1900, date->tm_mon + 1, date->tm_mday, date->tm_hour, date->tm_min,
					 RTCOM_EL_EVENT_GET_FIELD (ev, free_text));
			} else {
				fprintf (fd, "sms;submit;\"\";\"%s\";\"\";\"%4d.%2d.%2d %2d:%2d\";\"\";\"%s\"\n",
					 RTCOM_EL_EVENT_GET_FIELD (ev, remote_uid),
					 date->tm_year + 1900, date->tm_mon + 1, date->tm_mday, date->tm_hour, date->tm_min,
					 RTCOM_EL_EVENT_GET_FIELD (ev, free_text));
			}
		}
		rtcom_el_event_free (ev);
	}

	g_object_unref (iter);

	fclose (fd);

exit:
	osso_deinitialize (osso_context);
	return 0;
}
