#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <libosso.h>

#include <gtk/gtk.h>
#include <hildon/hildon.h>

#include <libebook/e-book.h>
#include <libosso-abook/osso-abook.h>
#include <rtcom-eventlogger/eventlogger.h>

#define SERVICE_ID "RTCOM_EL_SERVICE_SMS"
#define SMS_INBOUND "RTCOM_EL_EVENTTYPE_SMS_INBOUND"
#define SMS_OUTBOUND "RTCOM_EL_EVENTTYPE_SMS_OUTBOUND"
#define LOCAL_UID "ring/tel/ring"
#define MYSELF "<SelfHandle>"
#define IS_READ TRUE

typedef struct _MessageStruct MessageStruct;

struct _MessageStruct {
	char *service_id;
	char *event_type_id;
	time_t date;
	int is_read;
	char *local_uid;
	char *local_name;
	char *remote_uid;
	char *remote_name;
	char *remote_ebook_uid;
	char *group_uid;
	char *free_text;
};

gint message_compare_func(gconstpointer a, gconstpointer b)
{
	MessageStruct *a_data = (MessageStruct *)a;
	MessageStruct *b_data = (MessageStruct *)b;

	if (a_data->date > b_data->date)
		return 1;
	else if (a_data->date < b_data->date)
		return -1;
	else
		return 0;
}

int main (int argc, char **argv)
{
	osso_context_t *osso_context;
	EBook *ebook;
	EBookQuery *query;
	RTComEl *el = NULL;
	RTComElEvent *ev = NULL;
	GError *error = NULL;
	GList *contact, *c, *sms_list = NULL, *msg;
	char buf[512];
	FILE *fd;
	int lineno = 0;

	if (argc != 2) {
		printf ("Usage: smsimporter <input csv file>\n");
		printf ("  ex. smsimporter backup.csv\n");
		return 0;
	}

	hildon_gtk_init (&argc, &argv);

	/* Initialize osso */
	osso_context = osso_initialize ("smsimporter", "0.2", TRUE, NULL);
	if (osso_context == NULL) {
		g_critical ("Error initializing osso");
		return 1;
	}

	/* Init abook */
	if (!osso_abook_init (&argc, &argv, osso_context)) {
		g_critical ("Error initializing libosso-abook");
		goto exit;
	}

	ebook = e_book_new_system_addressbook (&error);
	if (!ebook) {
		g_warning ("Error opening system address book: %s", error->message);
		g_error_free (error);
		goto exit;
	}

	if (!e_book_open (ebook, TRUE, &error)) {
		g_warning ("Error opening system address book: %s", error->message);
		g_error_free (error);
		goto exit;
	}

	query = e_book_query_any_field_contains ("");

	if (!e_book_get_contacts (ebook, query, &contact, &error)) {
		g_warning ("Error getting contacts: %s", error->message);
		g_error_free (error);
		goto exit;
	}

	el = rtcom_el_new ();

	fd = fopen (argv[1], "r");
	if (!fd) {
		printf ("Error openning: %s\n", argv[1]);
		return 1;
	}

	while (fgets(buf, 512, fd)) {
		char *tmp, *event_type_id, *local_name, *remote_uid, *group_uid, *free_text;
		char delimiter[2];
		const char *remote_name, *remote_ebook_uid;
		int year = 0, month = 0, day = 0, hour = 0, min = 0, date_items = 0;
		char text[1561];
		struct tm date;
		unsigned int i = 0, skip = 0;
		MessageStruct *message;

		/* increment line number */
		lineno++;

		if (!strncmp(buf, "sms", 3)) {
			/* detect delimiter character */
			memcpy(delimiter, buf + 3, 2);
			if (delimiter == NULL)
				continue;

			/* add string termination to the end of delimiter */
			delimiter[1] = '\0';

			tmp = strtok (buf, delimiter);
			if (tmp == NULL)
				continue;

			for (i = 0; i < 7; i++) {
				tmp = strtok (NULL, delimiter);
				if (tmp == NULL)
					continue;

				switch (i) {
					case 0:
						if (!strcmp (tmp, "deliver")) {
							event_type_id = SMS_INBOUND;
							local_name = NULL;
							
						} else {
							event_type_id = SMS_OUTBOUND;
							local_name = MYSELF;
						}
						break;

					case 1:
						if (!strcmp (event_type_id, SMS_INBOUND)) {
							if (!strcmp (tmp, "\"\"")) {
								skip = 1;
								break;
							}
							remote_uid = tmp;

							/* remove apostrophes from start and end of the phone number */
							remote_uid++;
							remote_uid[strlen (remote_uid) - 1] = '\0';

							/* group uid is the last 7 numbers from phone number */
							if ((int) (strlen (remote_uid) - 7) >= 0) {
								group_uid = remote_uid + strlen (remote_uid) - 7;
							} else {
								group_uid = remote_uid;
							}
						}
						break;

					case 2:
						if (!strcmp (event_type_id, SMS_OUTBOUND)) {
							if (!strcmp (tmp, "\"\"")) {
								skip = 1;
								break;
							}
							remote_uid = tmp;

							/* remove apostrophes from start and end of the phone number */
							remote_uid++;
							remote_uid[strlen (remote_uid) - 1] = '\0';

							/* group uid is the last 7 numbers from phone number */
							if ((int) (strlen (remote_uid) - 7) >= 0)
								group_uid = remote_uid + strlen (remote_uid) - 7;
							else
								group_uid = remote_uid;
						}
						break;

					case 4:
						date_items = sscanf (tmp, "\"%4d.%2d.%2d %2d:%2d\"", &year, &month, &day, &hour, &min);
						if (date_items != 5)
							break;

						date.tm_year = year - 1900;
						date.tm_mon = month - 1;
						date.tm_mday = day;
						date.tm_hour = hour;
						date.tm_min = min;
						date.tm_sec = 0;
						date.tm_isdst = 0;
						break;

					case 6:
						free_text = tmp;

						/* remove apostrophes from start and end of the text of message */
						free_text++;
						free_text[strlen (free_text) - 2] = '\0';

						break;
				}
			}

			if (remote_uid == NULL || group_uid == NULL || date_items != 5 || free_text == NULL) {
				printf ("Error parsing line: %d\n", lineno);
				continue;
			}

			if (!skip) {
				const gchar *remote_name = NULL, *remote_ebook_uid = NULL;
				for (c = contact; c != NULL; c = c->next) {
					unsigned int num;

					for (num = E_CONTACT_FIRST_PHONE_ID; num <= E_CONTACT_LAST_PHONE_ID; num++) {
						const gchar *number = NULL;

						number = e_contact_get_const (E_CONTACT (c->data), num);
						if (!number)
							continue;

						//if (!strcmp (number, remote_uid)) {
						if (g_str_has_suffix (number, group_uid)) {
							remote_ebook_uid = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_UID);
							if (remote_ebook_uid) {
								remote_name = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_FULL_NAME);
								if (!remote_name) {
									remote_name = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_GIVEN_NAME);
									if (!remote_name) {
										remote_name = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_FAMILY_NAME);
										if (!remote_name) {
											remote_name = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_NICKNAME);
											if (!remote_name) {
												remote_name = e_contact_get_const (E_CONTACT (c->data), E_CONTACT_ORG);
											} else {
												remote_name = NULL;
											}
										}
									}
								}
							}

						}
					}
				}

				message = g_new0 (MessageStruct, 1);
				message->service_id = g_strdup (SERVICE_ID);
				message->event_type_id = g_strdup (event_type_id);
				message->date = mktime (&date);
				message->is_read = IS_READ;
				message->local_uid = g_strdup (LOCAL_UID);
				message->local_name = g_strdup (local_name);
				message->remote_uid = g_strdup (remote_uid);
				message->remote_name = g_strdup (remote_name);
				message->remote_ebook_uid = g_strdup (remote_ebook_uid);
				message->group_uid = g_strdup (group_uid);
				message->free_text = g_strdup (free_text);
				sms_list = g_list_append (sms_list, message);
			}
		}
	}

	fclose (fd);

	/* sort sms list */
	sms_list = g_list_sort (sms_list, message_compare_func);
	for (msg = sms_list; msg != NULL; msg = msg->next) {
		MessageStruct *message = msg->data;
		printf ("service_id:%s,event_type_id:%s,storage_time:%lu,start_time:%lu,end_time:%lu,is_read:%d,local_uid='%s',local_name='%s',remote_uid='%s',remote_name='%s',remote_ebook_uid='%s',free_text='%s',group_uid='%s'\n\n",
			message->service_id, message->event_type_id, message->date, message->date, message->date, message->is_read, message->local_uid, message->local_name,
			message->remote_uid, message->remote_name, message->remote_ebook_uid, message->free_text, message->group_uid);

			ev = rtcom_el_event_new ();
			RTCOM_EL_EVENT_SET_FIELD (ev, service, message->service_id);
			RTCOM_EL_EVENT_SET_FIELD (ev, event_type, message->event_type_id);
			RTCOM_EL_EVENT_SET_FIELD (ev, storage_time, message->date);
			RTCOM_EL_EVENT_SET_FIELD (ev, start_time, message->date);
			RTCOM_EL_EVENT_SET_FIELD (ev, end_time, message->date);
			RTCOM_EL_EVENT_SET_FIELD (ev, is_read, message->is_read);
			RTCOM_EL_EVENT_SET_FIELD (ev, local_uid, message->local_uid);
			RTCOM_EL_EVENT_SET_FIELD (ev, local_name, message->local_name);
			RTCOM_EL_EVENT_SET_FIELD (ev, remote_uid, message->remote_uid);
			RTCOM_EL_EVENT_SET_FIELD (ev, remote_name, message->remote_name);
			RTCOM_EL_EVENT_SET_FIELD (ev, remote_ebook_uid, message->remote_ebook_uid);
			RTCOM_EL_EVENT_SET_FIELD (ev, group_uid, message->group_uid);
			RTCOM_EL_EVENT_SET_FIELD (ev, free_text, message->free_text);
			rtcom_el_add_event (el, ev, NULL);
			rtcom_el_event_free (ev);
	}

exit:
	osso_deinitialize (osso_context);
	return 0;
}
